package net.jieyuu.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import net.jieyuu.enums.BizCodeEnum;
import net.jieyuu.enums.SendCodeEnum;
import net.jieyuu.interceptor.LoginInterceptor;
import net.jieyuu.model.LoginUser;
import net.jieyuu.model.UserDO;
import net.jieyuu.mapper.UserMapper;
import net.jieyuu.request.UserLoginRequest;
import net.jieyuu.request.UserRegisterRequest;
import net.jieyuu.service.NotifyService;
import net.jieyuu.service.UserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.jieyuu.utils.CommonUtil;
import net.jieyuu.utils.JWTUtil;
import net.jieyuu.utils.JsonData;
import net.jieyuu.vo.UserVO;
import org.apache.commons.codec.digest.Md5Crypt;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jieyuu
 * @since 2024-02-12
 */
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, UserDO> implements UserService {

    @Autowired
    private NotifyService notifyService;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 用户注册
     * 邮箱验证码验证
     * 密码加密 TODO
     * 账号唯一性检查 TODO
     * 插入数据库
     * 新注册用户福利发放 TODO
     *
     * @param registerRequest
     * @return
     */
    @Override
    public JsonData register(UserRegisterRequest registerRequest) {
        boolean checkCode = false;
        //校验验证码
        if (StringUtils.isNotBlank(registerRequest.getMail())) {
            checkCode = notifyService.checkCode(SendCodeEnum.USER_REGISTER, registerRequest.getMail(), registerRequest.getCode());
        }

        if (!checkCode) {
            //验证码错误
            return JsonData.buildResult(BizCodeEnum.CODE_CAPTCHA);
        }

        UserDO userDO = new UserDO();
        BeanUtils.copyProperties(registerRequest, userDO);

        //设置创建时间
        userDO.setCreateTime(new Date());

        //设置密码 加密
//        userDO.setPwd(userRegisterRequest.getPwd());
        //生成密钥 盐e
        userDO.setSecret("$1$" + CommonUtil.getStringNumRandom(8));

        //密码加盐处理
        String cryptPwd = Md5Crypt.md5Crypt(userDO.getPwd().getBytes(), userDO.getSecret());
        userDO.setPwd(cryptPwd);

        //账号唯一性检查 todo
        if (checkUnique(userDO.getMail())) {
            //插入数据库
            int rows = userMapper.insert(userDO);
            log.info("rows:{},注册成功{}", rows, userDO.toString());

            //新用户创建成功，初始化，发福利 todo
            userRegisterInitTask(userDO);
            return JsonData.buildSuccess();

        } else {
            return JsonData.buildResult(BizCodeEnum.ACCOUNT_REPEAT);
        }
    }


    /**
     * 校验用户账号唯一
     *
     * @param mail
     * @return
     */
    private boolean checkUnique(String mail) {
        //todo 高并发的思考
        QueryWrapper queryWrapper = new QueryWrapper<UserDO>().eq("mail", mail);

        List<UserDO> list = userMapper.selectList(queryWrapper);

        return list.size() > 0 ? false : true;

    }

    /**
     * 用户注册，初始化福利信息 todo
     *
     * @param userDO
     */
    private void userRegisterInitTask(UserDO userDO) {

    }

    /**
     * 用户登陆注册
     * <p>
     * 判断用户注册情况
     * 使用密钥+密文加密，判断密文匹配
     *
     * @param loginRequest
     * @return
     */
    @Override
    public JsonData login(UserLoginRequest loginRequest) {
        List<UserDO> userDOList = userMapper.selectList(new QueryWrapper<UserDO>().eq("mail", loginRequest.getMail()));

        if (userDOList != null && userDOList.size() == 1) {
            //已经注册
            UserDO userDO = userDOList.get(0);
            String cryptPwd = Md5Crypt.md5Crypt(loginRequest.getPwd().getBytes(), userDO.getSecret());
            if (cryptPwd.equals(userDO.getPwd())) {
                //登陆成功
                //生成token令牌并且返回
                LoginUser loginUser = LoginUser.builder().build();
                BeanUtils.copyProperties(userDO, loginUser);
                //accessToken
                String accessToken = JWTUtil.geneJsonWebToken(loginUser);
                // accessToken过期时间
                //UUID生成一个token
                String refreshToken = CommonUtil.generateUUID();
//                redisTemplate.opsForValue().set(refreshToken, "1", 1000 * 60 * 60 * 24 * 30);


                return JsonData.buildSuccess(accessToken);
            } else {
                return JsonData.buildResult(BizCodeEnum.ACCOUNT_PWD_ERROR);
            }

        } else {
            //未注册
            //这里选择 账号或密码错误，防止被爆破
            return JsonData.buildResult(BizCodeEnum.ACCOUNT_PWD_ERROR);
        }
    }

    @Override
    public UserVO findUserDetail() {
        LoginUser loginUser = LoginInterceptor.threadLocal.get();

        UserDO userDO = userMapper.selectOne(new QueryWrapper<UserDO>().eq("id", loginUser.getId()));

        UserVO userVO = new UserVO();
        BeanUtils.copyProperties(userDO, userVO);
        return userVO;
    }


}
